from base64 import b64encode
from io import BytesIO
from pathlib import Path

from PIL import Image as PILImage
from PIL import UnidentifiedImageError
from textual.app import ComposeResult
from textual.containers import Container, Horizontal, Vertical
from textual.screen import ModalScreen
from textual.widgets import DirectoryTree, Label

from oterm.app.widgets.image import IMAGE_EXTENSIONS, Image, ImageDirectoryTree


class ImageSelect(ModalScreen[tuple[Path, str]]):
    BINDINGS = [
        ("escape", "cancel", "Cancel"),
    ]

    def action_cancel(self) -> None:
        self.dismiss()

    async def on_mount(self) -> None:
        dt = self.query_one(ImageDirectoryTree)
        dt.show_guides = False

    async def on_directory_tree_file_selected(
        self, ev: DirectoryTree.FileSelected
    ) -> None:
        try:
            buffer = BytesIO()
            image = PILImage.open(ev.path)
            if image.mode != "RGB":
                image = image.convert("RGB")
            image.save(buffer, format="JPEG")
            b64 = b64encode(buffer.getvalue()).decode("utf-8")
            self.dismiss((ev.path, b64))
        except UnidentifiedImageError:
            self.dismiss()

    async def on_tree_node_highlighted(self, ev: DirectoryTree.NodeHighlighted) -> None:
        path = ev.node.data.path
        if path.suffix in IMAGE_EXTENSIONS:
            image = self.query_one(Image)
            image.path = path.as_posix()

    def compose(self) -> ComposeResult:
        with Container(id="image-select-container"):
            with Horizontal():
                with Vertical(id="image-directory-tree"):
                    yield Label("Select an image:", classes="title")
                    yield ImageDirectoryTree("./")
                with Container(id="image-preview"):
                    yield Image(id="image")



from rich.syntax import Syntax
from textual.app import ComposeResult
from textual.widgets import Markdown, Static
from textual.widgets._markdown import MarkdownFence as BaseMarkdownFence


class MarkdownFence(BaseMarkdownFence):
    """A MarkdownFence that responds to the app's dark mode."""

    def __init__(self, markdown: Markdown, code: str, lexer: str) -> None:
        super().__init__(markdown, code, lexer)
        self.theme = "solarized-dark" if self.app.dark else "solarized-light"

    def _retheme(self) -> None:
        """Swap between a dark and light theme when the mode changes."""
        self.theme = "solarized-dark" if self.app.dark else "solarized-light"
        code_block = self.query_one(".code-block", Static)
        code_block.renderable = Syntax(
            self.code,
            lexer=self.lexer,
            word_wrap=False,
            indent_guides=True,
            padding=(1, 2),
            theme=self.theme,
        )

    def on_mount(self) -> None:
        """Watch app theme switching."""
        self.watch(self.app, "dark", self._retheme)

    def compose(self) -> ComposeResult:
        yield Static(
            Syntax(
                self.code,
                lexer=self.lexer,
                word_wrap=False,
                indent_guides=True,
                padding=(1, 2),
                theme=self.theme,
            ),
            expand=True,
            shrink=False,
            classes="code-block",
        )



from pathlib import Path
from typing import Awaitable, Callable

import aiosqlite


async def add_format_to_chat(db_path: Path) -> None:
    async with aiosqlite.connect(db_path) as connection:
        try:
            await connection.executescript(
                """
                ALTER TABLE chat ADD COLUMN format TEXT;
                """
            )
        except aiosqlite.OperationalError:
            pass


upgrades: list[tuple[str, list[Callable[[Path], Awaitable[None]]]]] = [
    ("0.1.11", [add_format_to_chat]),
    ("0.1.13", [add_format_to_chat]),
]



import json
from enum import Enum
from pathlib import Path
from typing import Literal

import pyperclip
from textual import on
from textual.app import ComposeResult
from textual.containers import Horizontal, Vertical
from textual.events import Click
from textual.reactive import reactive
from textual.widget import Widget
from textual.widgets import (
    LoadingIndicator,
    Markdown,
    Static,
    TabbedContent,
)

from oterm.app.chat_rename import ChatRename
from oterm.app.widgets.image import ImageAdded
from oterm.app.widgets.prompt import FlexibleInput
from oterm.ollama import OllamaLLM


class Author(Enum):
    USER = "me"
    OLLAMA = "ollama"


class ChatContainer(Widget):
    ollama = OllamaLLM()
    messages: reactive[list[tuple[Author, str]]] = reactive([])
    chat_name: str
    system: str | None
    template: str | None
    format: Literal["json"] | None
    images: list[tuple[Path, str]] = []

    BINDINGS = [
        ("ctrl+r", "rename_chat", "rename chat"),
        ("ctrl+x", "forget_chat", "forget chat"),
    ]

    def __init__(
        self,
        *children: Widget,
        db_id: int,
        chat_name: str,
        model: str = "nous-hermes:13b",
        context: list[int] = [],
        messages: list[tuple[Author, str]] = [],
        system: str | None = None,
        template: str | None = None,
        format: Literal["json"] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(*children, **kwargs)
        self.ollama = OllamaLLM(
            model=model,
            context=context,
            template=template,
            system=system,
            format=format,
        )  # We do this to reset the context
        self.chat_name = chat_name
        self.db_id = db_id
        self.messages = messages
        self.system = system
        self.template = template
        self.format = format

    def on_mount(self) -> None:
        self.query_one("#prompt").focus()
        message_container = self.query_one("#messageContainer")
        for author, message in self.messages:
            chat_item = ChatItem()
            chat_item.text = message
            chat_item.author = author
            message_container.mount(chat_item)
        message_container.scroll_end()

    @on(FlexibleInput.Submitted)
    async def on_submit(self, event: FlexibleInput.Submitted) -> None:
        message = event.value
        input = event.input
        message_container = self.query_one("#messageContainer")

        if not message.strip():
            input.clear()
            input.focus()
            return

        input.clear()
        input.disabled = True
        self.messages.append((Author.USER, message))
        chat_item = ChatItem()
        chat_item.text = message
        chat_item.author = Author.USER
        message_container.mount(chat_item)

        chat_item = ChatItem()
        chat_item.author = Author.OLLAMA
        message_container.mount(chat_item)
        loading = LoadingIndicator()
        message_container.mount(loading)
        message_container.scroll_end()

        response = ""
        async for text in self.ollama.stream(message, [img for _, img in self.images]):
            response = text
            chat_item.text = text
            message_container.scroll_end()
        self.messages.append((Author.OLLAMA, response))
        self.images = []
        loading.remove()
        input.disabled = False
        input.focus()

        # Save to db
        await self.app.store.save_context(  # type: ignore
            id=self.db_id,
            context=json.dumps(self.ollama.context),
        )
        await self.app.store.save_message(  # type: ignore
            chat_id=self.db_id,
            author=Author.USER.value,
            text=message,
        )
        await self.app.store.save_message(  # type: ignore
            chat_id=self.db_id,
            author=Author.OLLAMA.value,
            text=response,
        )

    async def action_rename_chat(self) -> None:
        async def on_chat_rename(name: str) -> None:
            tabs = self.app.query_one(TabbedContent)
            await self.app.store.rename_chat(self.db_id, name)
            tabs.get_tab(f"chat-{self.db_id}").update(name)

        screen = ChatRename()
        screen.old_name = self.chat_name
        self.app.push_screen(screen, on_chat_rename)

    async def action_forget_chat(self) -> None:
        tabs = self.app.query_one(TabbedContent)
        await self.app.store.delete_chat(self.db_id)
        tabs.remove_pane(tabs.active)

    @on(ImageAdded)
    def on_image_added(self, ev: ImageAdded) -> None:
        self.images.append((ev.path, ev.image))
        message_container = self.query_one("#messageContainer")
        notification = Notification()
        notification.message = f"Image {ev.path} added."
        message_container.mount(notification)
        message_container.scroll_end()

    def compose(self) -> ComposeResult:
        with Vertical():
            yield Static(f"model: {self.ollama.model}", id="info")
            yield Vertical(id="messageContainer")
            yield FlexibleInput("", id="prompt", classes="singleline")


class ChatItem(Widget):
    text: reactive[str] = reactive("")
    author: Author

    @on(Click)
    async def on_click(self, event: Click) -> None:
        pyperclip.copy(self.text)
        widgets = self.query(".text")
        for widget in widgets:
            widget.styles.animate("opacity", 0.5, duration=0.1)
            widget.styles.animate("opacity", 1.0, duration=0.1, delay=0.1)

    async def watch_text(self, text: str) -> None:
        text = self.text
        try:
            jsn = json.loads(text)
            if isinstance(jsn, dict):
                text = f"```json\n{self.text}\n```"
        except json.JSONDecodeError:
            pass

        txt_widget = self.query_one(".text", Markdown)
        await txt_widget.update(text)

    def compose(self) -> ComposeResult:
        """A chat item."""
        with Horizontal(classes=f"{self.author.name} chatItem"):
            yield Static(self.author.value, classes="author", markup=False)
            yield Markdown(self.text, classes="text txt")


class Notification(Widget):
    message: reactive[str] = reactive("")

    def compose(self) -> ComposeResult:
        yield Static(self.message, classes="notification")



import sys
from pathlib import Path


def get_data_dir() -> Path:
    """
    Get the user data directory for the current system platform.

    Linux: ~/.local/share/oterm
    macOS: ~/Library/Application Support/oterm
    Windows: C:/Users/<USER>/AppData/Roaming/oterm

    :return: User Data Path
    :rtype: str
    """
    home = Path.home()

    system_paths = {
        "win32": home / "AppData/Roaming/oterm",
        "linux": home / ".local/share/oterm",
        "darwin": home / "Library/Application Support/oterm",
    }

    data_path = system_paths[sys.platform]
    return data_path


def semantic_version_to_int(version: str) -> int:
    """
    Convert a semantic version string to an integer.

    :param version: Semantic version string
    :type version: str
    :return: Integer representation of semantic version
    :rtype: int
    """
    major, minor, patch = version.split(".")
    major = int(major) << 16
    minor = int(minor) << 8
    patch = int(patch)
    return major + minor + patch


def int_to_semantic_version(version: int) -> str:
    """
    Convert an integer to a semantic version string.

    :param version: Integer representation of semantic version
    :type version: int
    :return: Semantic version string
    :rtype: str
    """
    major = version >> 16
    minor = (version >> 8) & 255
    patch = version & 255
    return f"{major}.{minor}.{patch}"



from textual.events import Key
from textual.widgets import TextArea as BaseTextArea


class TextArea(BaseTextArea):
    """A TextArea that responds to the app's dark mode.
    This might be potentially fixed after https://github.com/Textualize/textual/issues/3997 is resolved.
    In addition we ignore the tab key to allow it to move along the focus chain.
    """

    def _retheme(self) -> None:
        """Swap between a dark and light theme when the mode changes."""
        self.theme = "vscode_dark" if self.app.dark else "github_light"

    def on_mount(self) -> None:
        """Watch app theme switching."""
        self.watch(self.app, "dark", self._retheme)

    async def _on_key(self, event: Key) -> None:
        """Allow tab to move along the focus chain."""
        if event.key == "tab":
            event.prevent_default()
            self.screen.focus_next()



import json
from ast import literal_eval

from rich.text import Text
from textual.app import ComposeResult
from textual.containers import Container, Horizontal, Vertical
from textual.css.query import NoMatches
from textual.reactive import reactive
from textual.screen import ModalScreen
from textual.widgets import Button, Checkbox, Label, OptionList, Pretty

from oterm.app.widgets.text_area import TextArea
from oterm.ollama import OllamaAPI


class ModelSelection(ModalScreen[str]):
    api = OllamaAPI()
    models = []
    models_info: dict[str, dict] = {}

    model_name: reactive[str] = reactive("")
    tag: reactive[str] = reactive("")
    bytes: reactive[int] = reactive(0)
    model_info: reactive[dict[str, str]] = reactive({}, layout=True)
    template: reactive[str] = reactive("")
    system: reactive[str] = reactive("")
    params: reactive[list[tuple[str, str]]] = reactive([], layout=True)
    json_format: reactive[bool] = reactive(False)

    last_highlighted_index = None

    BINDINGS = [
        ("escape", "cancel", "Cancel"),
        ("enter", "create", "Create"),
    ]

    def action_cancel(self) -> None:
        self.dismiss()

    def action_create(self) -> None:
        self._create_chat()

    def _create_chat(self) -> None:
        model = f"{self.model_name}:{self.tag}"
        template = self.query_one(".template", TextArea).text
        template = template if template != self.model_info.get("template", "") else None
        system = self.query_one(".system", TextArea).text
        system = system if system != self.model_info.get("system", "") else None
        jsn = self.query_one(".json-format", Checkbox).value
        result = json.dumps(
            {
                "name": model,
                "template": template,
                "system": system,
                "format": "json" if jsn else None,
            }
        )
        self.dismiss(result)

    async def on_mount(self) -> None:
        self.models = await self.api.get_models()
        models = [model["name"] for model in self.models]
        for model in models:
            info = await self.api.get_model_info(model)
            for key in ["modelfile", "license"]:
                if key in info.keys():
                    del info[key]
            self.models_info[model] = info
        option_list = self.query_one("#model-select", OptionList)
        option_list.clear_options()
        for model in models:
            option_list.add_option(item=self.model_option(model))
        option_list.highlighted = self.last_highlighted_index

    def on_option_list_option_selected(self, option: OptionList.OptionSelected) -> None:
        self._create_chat()

    def on_option_list_option_highlighted(
        self, option: OptionList.OptionHighlighted
    ) -> None:
        model = option.option.prompt
        model_meta = next((m for m in self.models if m["name"] == str(model)), None)
        if model_meta:
            name, tag = model_meta["name"].split(":")
            self.model_name = name
            self.tag = tag
            self.bytes = model_meta["size"]

            self.model_info = self.models_info[model_meta["name"]]

        # Now that there is a model selected we can create the chat.
        create_button = self.query_one("#create-btn", Button)
        create_button.disabled = False
        ModelSelection.last_highlighted_index = option.option_index

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.name == "create":
            self._create_chat()
        else:
            self.dismiss()

    @staticmethod
    def model_option(model: str) -> Text:
        return Text(model)

    def watch_name(self, name: str) -> None:
        try:
            widget = self.query_one(".name", Label)
            widget.update(f"Name: {self.model_name}")
        except NoMatches:
            pass

    def watch_tag(self, tag: str) -> None:
        try:
            widget = self.query_one(".tag", Label)
            widget.update(f"Tag: {self.tag}")
        except NoMatches:
            pass

    def watch_bytes(self, size: int) -> None:
        try:
            widget = self.query_one(".size", Label)
            widget.update(f"Size: {(self.bytes / 1.0e9):.2f} GB")
        except NoMatches:
            pass

    def watch_model_info(self, model_info: dict[str, str]) -> None:
        self.template = model_info.get("template", "")
        self.system = model_info.get("system", "")
        params = model_info.get("parameters", "")
        lines = params.split("\n")
        params = []
        for line in lines:
            if line:
                key, value = line.split(maxsplit=1)
                try:
                    value = literal_eval(value)
                except (SyntaxError, ValueError):
                    pass
                params.append((key, value))
        self.params = params

        try:
            widget = self.query_one(".parameters", Pretty)
            widget.update(self.params)
            widget = self.query_one(".template", TextArea)
            widget.clear()
            widget.load_text(self.template)
            widget = self.query_one(".system", TextArea)
            widget.load_text(self.system)
        except NoMatches:
            pass

    def compose(self) -> ComposeResult:
        with Container(id="model-select-container"):
            yield Label("Select a model:", classes="title")
            with Horizontal():
                with Vertical():
                    yield OptionList(id="model-select")
                    with Vertical(id="model-details"):
                        yield Label("Model info:", classes="title")
                        yield Label("", classes="name")
                        yield Label("", classes="tag")
                        yield Label("", classes="size")
                with Vertical():
                    yield Label("Template:", classes="title")
                    yield TextArea(classes="template log")
                    yield Label("System:", classes="title")
                    yield TextArea("", classes="system log")
                    yield Label("Parameters:", classes="title")
                    yield Pretty("", classes="parameters")
                    yield Label("Format", classes="title")
                    yield Checkbox("JSON output", value=False, classes="json-format")

            with Horizontal(classes="button-container"):
                yield Button(
                    "Create",
                    id="create-btn",
                    name="create",
                    disabled=True,
                    variant="primary",
                )
                yield Button("Cancel", name="cancel")



import json
from importlib import metadata
from pathlib import Path
from typing import Literal

import aiosqlite
from packaging.version import parse

from oterm.app.widgets.chat import Author
from oterm.store.chat import queries as chat_queries
from oterm.store.setup import queries as setup_queries
from oterm.store.upgrades import upgrades
from oterm.utils import get_data_dir, int_to_semantic_version, semantic_version_to_int


class Store(object):
    db_path: Path

    @classmethod
    async def create(cls) -> "Store":
        self = Store()
        data_path = get_data_dir()
        data_path.mkdir(parents=True, exist_ok=True)
        self.db_path = data_path / "store.db"

        if not self.db_path.exists():
            # Create tables and set user_version
            async with aiosqlite.connect(self.db_path) as connection:
                await setup_queries.create_chat_table(connection)  # type: ignore
                await setup_queries.create_message_table(connection)  # type: ignore
                await self.set_user_version(metadata.version("oterm"))
        else:
            # Upgrade database
            current_version: str = metadata.version("oterm")
            db_version = await self.get_user_version()
            for version, steps in upgrades:
                if parse(current_version) >= parse(version) and parse(version) > parse(
                    db_version
                ):
                    for step in steps:
                        await step(self.db_path)
            await self.set_user_version(current_version)
        return self

    async def get_user_version(self) -> str:
        async with aiosqlite.connect(self.db_path) as connection:
            res = await setup_queries.get_user_version(connection)  # type: ignore
            return int_to_semantic_version(res[0][0])

    async def set_user_version(self, version: str) -> None:
        async with aiosqlite.connect(self.db_path) as connection:
            await connection.execute(
                f"PRAGMA user_version = {semantic_version_to_int(version)};"
            )

    async def save_chat(
        self,
        id: int | None,
        name: str,
        model: str,
        context: str,
        template: str | None,
        system: str | None,
        format: str | None,
    ) -> int:
        async with aiosqlite.connect(self.db_path) as connection:
            res: list[tuple[int]] = await chat_queries.save_chat(  # type: ignore
                connection,
                id=id,
                name=name,
                model=model,
                context=context,
                template=template,
                system=system,
                format=format,
            )

            await connection.commit()
            return res[0][0]

    async def save_context(self, id: int, context: str) -> None:
        async with aiosqlite.connect(self.db_path) as connection:
            await chat_queries.save_context(  # type: ignore
                connection,
                id=id,
                context=context,
            )
            await connection.commit()

    async def rename_chat(self, id: int, name: str) -> None:
        async with aiosqlite.connect(self.db_path) as connection:
            await chat_queries.rename_chat(  # type: ignore
                connection,
                id=id,
                name=name,
            )
            await connection.commit()

    async def get_chats(
        self,
    ) -> list[
        tuple[int, str, str, list[int], str | None, str | None, Literal["json"] | None]
    ]:
        async with aiosqlite.connect(self.db_path) as connection:
            chats = await chat_queries.get_chats(connection)  # type: ignore
            chats = [
                (id, name, model, json.loads(context), template, system, format)
                for id, name, model, context, template, system, format in chats
            ]
            return chats

    async def get_chat(
        self, id
    ) -> tuple[
        int, str, str, list[int], str | None, str | None, Literal["json"] | None
    ] | None:
        async with aiosqlite.connect(self.db_path) as connection:
            chat = await chat_queries.get_chat(connection, id=id)  # type: ignore
            if chat:
                chat = chat[0]
                id, name, model, context, template, system, format = chat
                context = json.loads(context)
                return id, name, model, context, template, system, format

    async def delete_chat(self, id: int) -> None:
        async with aiosqlite.connect(self.db_path) as connection:
            await chat_queries.delete_chat(connection, id=id)  # type: ignore
            await connection.commit()

    async def save_message(self, chat_id: int, author: str, text: str) -> None:
        async with aiosqlite.connect(self.db_path) as connection:
            await chat_queries.save_message(  # type: ignore
                connection,
                chat_id=chat_id,
                author=author,
                text=text,
            )
            await connection.commit()

    async def get_messages(self, chat_id: int) -> list[tuple[Author, str]]:
        async with aiosqlite.connect(self.db_path) as connection:
            messages = await chat_queries.get_messages(connection, chat_id=chat_id)  # type: ignore
            messages = [(Author(author), text) for author, text in messages]
            return messages



from pathlib import Path
from typing import Iterable

from PIL import Image as PILImage
from PIL import UnidentifiedImageError
from rich_pixels import Pixels
from textual.message import Message
from textual.reactive import reactive
from textual.widget import Widget
from textual.widgets import DirectoryTree

IMG_MAX_SIZE = 80
IMAGE_EXTENSIONS = PILImage.registered_extensions()


class ImageAdded(Message):
    def __init__(self, path: Path, image: str) -> None:
        self.path = path
        self.image = image
        super().__init__()


class Image(Widget):
    path: reactive[str] = reactive("")

    def __init__(self, id="", classes="") -> None:
        self.pixels = None
        super().__init__(id=id, classes=classes)

    def watch_path(self, path: str) -> None:
        if path:
            try:
                with PILImage.open(path) as img:
                    max_size = max(img.width, img.height)
                    width = int(img.width / max_size * IMG_MAX_SIZE)
                    height = int(img.height / max_size * IMG_MAX_SIZE)

                    self.set_styles(
                        f"""
                        width: {width * 2};
                        height: {height};
                        padding:1;
                        """
                    )
                    self.pixels = Pixels.from_image_path(path, (width, height))
            except UnidentifiedImageError:
                self.pixels = None
        else:
            self.pixels = None

    def render(self):
        if self.pixels:
            return self.pixels
        return ""


class ImageDirectoryTree(DirectoryTree):
    def filter_paths(self, paths: Iterable[Path]) -> Iterable[Path]:
        return [
            path for path in paths if path.suffix in IMAGE_EXTENSIONS or path.is_dir()
        ]



import json
from typing import Any, AsyncGenerator, Literal

import httpx

from oterm.config import envConfig


class OllamaError(Exception):
    pass


class OllamaConnectError(OllamaError):
    def __init__(self) -> None:
        super().__init__(
            f"Failed to connect to Ollama server running at {envConfig.OLLAMA_URL}. "
            "You can set OLLAMA_URL if you want to use a different server."
        )


class OllamaLLM:
    def __init__(
        self,
        model="nous-hermes:13b",
        template: str | None = None,
        system: str | None = None,
        context: list[int] = [],
        format: Literal["json"] | None = None,
    ):
        self.model = model
        self.template = template
        self.system = system
        self.context = context
        self.format = format

    async def completion(self, prompt: str, images: list[str] = []) -> str:
        response = ""
        context = []
        async for text, ctx in self._agenerate(
            prompt=prompt,
            context=self.context,
            images=images,
        ):
            response = text
            context = ctx
        self.context = context
        return response

    async def stream(
        self, prompt: str, images: list[str] = []
    ) -> AsyncGenerator[str, Any]:
        context = []

        async for text, ctx in self._agenerate(
            prompt=prompt,
            context=self.context,
            images=images,
        ):
            context = ctx
            yield text

        self.context = context

    async def _agenerate(
        self, prompt: str, context: list[int], images: list[str] = []
    ) -> AsyncGenerator[tuple[str, list[int]], Any]:
        client = httpx.AsyncClient(verify=envConfig.OTERM_VERIFY_SSL)
        jsn = {
            "model": self.model,
            "prompt": prompt,
            "context": context,
        }
        if self.system:
            jsn["system"] = self.system
        if self.template:
            jsn["template"] = self.template
        if self.format:
            jsn["format"] = self.format
        if images:
            jsn["images"] = images
        res = ""

        try:
            async with client.stream(
                "POST", f"{envConfig.OLLAMA_URL}/generate", json=jsn, timeout=None
            ) as response:
                async for line in response.aiter_lines():
                    body = json.loads(line)
                    res += body.get("response", "")
                    yield res, []
                    if "error" in body:
                        raise OllamaError(body["error"])

                    if body.get("done", False):
                        yield res, body["context"]
        except httpx.ConnectError:
            raise OllamaConnectError()


class OllamaAPI:
    async def get_models(self) -> list[dict[str, Any]]:
        client = httpx.AsyncClient(verify=envConfig.OTERM_VERIFY_SSL)
        try:
            response = await client.get(f"{envConfig.OLLAMA_URL}/tags")
        except httpx.ConnectError:
            raise OllamaConnectError()
        return response.json().get("models", []) or []

    async def get_model_info(self, model: str) -> dict[str, Any]:
        client = httpx.AsyncClient(verify=envConfig.OTERM_VERIFY_SSL)
        try:
            response = await client.post(
                f"{envConfig.OLLAMA_URL}/show", json={"name": model}
            )
        except httpx.ConnectError:
            raise OllamaConnectError()
        if response.json().get("error"):
            raise OllamaError(response.json()["error"])
        return response.json()

    async def pull_model(self, model: str) -> None:
        client = httpx.AsyncClient()
        async with client.stream(
            "POST", f"{envConfig.OLLAMA_URL}/pull", json={"name": model}, timeout=None
        ) as response:
            async for line in response.aiter_lines():
                body = json.loads(line)
                if "error" in body:
                    raise OllamaError(body["error"])
                if body.get("status", "") == "success":
                    return
